@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2018 Meltytech, LLC
+ * Copyright (c) 2011-2019 Meltytech, LLC
  *
  * GL shader based on BSD licensed code from Peter Bengtsson:
  * http://www.fourcc.org/source/YUV420P-OpenGL-GLSLang.c
@@ -484,10 +484,23 @@ void GLWidget::mouseMoveEvent(QMouseEvent* event)
     }
     if (!(event->buttons() & Qt::LeftButton))
         return;
+    if (m_dragStart.isNull())
+        return;
     if ((event->pos() - m_dragStart).manhattanLength() < QApplication::startDragDistance())
         return;
-    if (!MLT.producer() || !MLT.isSeekableClip())
+    // Reset the drag point to prevent repeating drag actions.
+    m_dragStart.setX(0);
+    m_dragStart.setY(0);
+    if (!MLT.producer())
         return;
+    if (MLT.isMultitrack() || MLT.isPlaylist()) {
+        MAIN.showStatusMessage(tr("You cannot drag from Project."));
+        return;
+    } else if (!MLT.isSeekableClip()) {
+        MAIN.showStatusMessage(tr("You cannot drag a non-seekable source"));
+        return;
+    }
+
     QDrag *drag = new QDrag(this);
     QMimeData *mimeData = new QMimeData;
     mimeData->setData(Mlt::XmlMimeType, MLT.XML().toUtf8());
@@ -509,6 +522,14 @@ void GLWidget::keyPressEvent(QKeyEvent* event)
     MAIN.keyPressEvent(event);
 }
 
+bool GLWidget::event(QEvent* event)
+{
+    bool result = QQuickWidget::event(event);
+    if (event->type() == QEvent::PaletteChange && m_sharedFrame.is_valid())
+        onFrameDisplayed(m_sharedFrame);
+    return result;
+}
+
 void GLWidget::createThread(RenderThread **thread, thread_function_t function, void *data)
 {
 #ifdef Q_OS_WIN
@@ -643,8 +664,8 @@ int GLWidget::reconfigure(bool isMulti)
                 m_consumer->set("0.progressive", property("progressive").toBool());
             m_consumer->set("0.rescale", property("rescale").toString().toLatin1().constData());
             m_consumer->set("0.deinterlace_method", property("deinterlace_method").toString().toLatin1().constData());
-            m_consumer->set("0.buffer", 25);
-            m_consumer->set("0.prefill", 1);
+            m_consumer->set("0.buffer", qMax(25, qRound(profile().fps())));
+            m_consumer->set("0.prefill", qMax(1, qRound(profile().fps() / 25.0)));
             if (property("keyer").isValid())
                 m_consumer->set("0.keyer", property("keyer").toInt());
         }
@@ -653,8 +674,8 @@ int GLWidget::reconfigure(bool isMulti)
                 m_consumer->set("progressive", property("progressive").toBool());
             m_consumer->set("rescale", property("rescale").toString().toLatin1().constData());
             m_consumer->set("deinterlace_method", property("deinterlace_method").toString().toLatin1().constData());
-            m_consumer->set("buffer", 25);
-            m_consumer->set("prefill", 1);
+            m_consumer->set("buffer", qMax(25, qRound(profile().fps())));
+            m_consumer->set("prefill", qMax(1, qRound(profile().fps() / 25.0)));
             if (property("keyer").isValid())
                 m_consumer->set("keyer", property("keyer").toInt());
         }
