@@ -341,6 +341,32 @@ void KeyframesModel::addKeyframe(int parameterIndex, double value, int position,
     }
 }
 
+void KeyframesModel::addKeyframe(int parameterIndex, int position)
+{
+    if (m_filter && parameterIndex < m_propertyNames.count()) {
+        QString name = m_propertyNames[parameterIndex];
+        // Get the value from the existing position.
+        double value = m_filter->getDouble(name, position);
+        Mlt::Animation anim = m_filter->getAnimation(name);
+        if (anim.is_valid() && !anim.is_key(position)) {
+            mlt_keyframe_type keyframeType = m_filter->getKeyframeType(anim, position, mlt_keyframe_type(-1));
+            // Simply adding a keyframe does not change the current value of
+            // the filter parameter. So, no need to trigger a bunch of signals
+            // and refresh the consumer. Besides, refreshing the consumer is
+            // some how causing the player to seek ahead by one frame inadvertently
+            // such that changing the parameter value causes the addition of a
+            // keyframe just after this one. MLT.refreshConsumer() with
+            // frame-dropping enabled may have dropped video of the most recent
+            // frame from the producer, but Shotcut does not know about it
+            // because it did not receive a "consumer-frame-show" event for it.
+            m_filter->blockSignals(true);
+            m_filter->set(name, value, position, keyframeType);
+            m_filter->blockSignals(false);
+            onFilterChanged(name);
+        }
+    }
+}
+
 void KeyframesModel::setKeyframe(int parameterIndex, double value, int position, KeyframesModel::InterpolationType type)
 {
     if (m_filter && parameterIndex < m_propertyNames.count()) {
@@ -355,6 +381,16 @@ void KeyframesModel::setKeyframe(int parameterIndex, double value, int position,
     }
 }
 
+bool KeyframesModel::isKeyframe(int parameterIndex, int position)
+{
+    if (m_filter && parameterIndex < m_propertyNames.count()) {
+        QString name = m_propertyNames[parameterIndex];
+        Mlt::Animation anim = m_filter->getAnimation(name);
+        return anim.is_valid() && anim.is_key(position);
+    }
+    return false;
+}
+
 void KeyframesModel::reload()
 {
     beginResetModel();
@@ -404,10 +440,9 @@ void KeyframesModel::onFilterInChanged(int delta)
         if (count > 0) {
             Mlt::Animation animation = m_filter->getAnimation(m_propertyNames[parameterIndex]);
             if (animation.is_valid()) {
-                int filterDuration = m_filter->duration();
                 for (int keyframeIndex = 0; keyframeIndex < count;) {
                     int newFrame = animation.key_get_frame(keyframeIndex) - delta;
-                    if (newFrame < 0 || newFrame >= filterDuration || animation.is_key(newFrame)) {
+                    if (animation.is_key(newFrame)) {
                         beginRemoveRows(index(parameterIndex), keyframeIndex, keyframeIndex);
                         animation.remove(animation.key_get_frame(keyframeIndex));
                         animation.interpolate();
@@ -434,10 +469,9 @@ void KeyframesModel::onFilterOutChanged(int delta)
         if (count > 0) {
             Mlt::Animation animation = m_filter->getAnimation(m_propertyNames[parameterIndex]);
             if (animation.is_valid()) {
-                int filterDuration = m_filter->duration();
                 for (int keyframeIndex = 0; keyframeIndex < count;) {
                     int frame = animation.key_get_frame(keyframeIndex);
-                    if (frame < 0 || frame >= filterDuration) {
+                    if (frame < 0) {
                         beginRemoveRows(index(parameterIndex), keyframeIndex, keyframeIndex);
                         animation.remove(animation.key_get_frame(keyframeIndex));
                         animation.interpolate();
